"""
List of adjectives
"""

ADJECTIVES = [
    "Accommodating",
    "Adventurous",
    "Affectionate",
    "Agreeable",
    "Alluring",
    "Altruistic",
    "Ambitious",
    "Amorous",
    "Amusing",
    "Appealing",
    "Ardent",
    "Attentive",
    "Attractive",
    "Audacious",
    "Authentic",
    "Awesome",
    "Balanced",
    "Benevolent",
    "Bold",
    "Brave",
    "Bright",
    "Capable",
    "Captivating",
    "Caring",
    "Charitable",
    "Charming",
    "Charming",
    "Cheerful",
    "Cheery",
    "Clever",
    "Comforting",
    "Comical",
    "Committed",
    "Compassionate",
    "Concerned",
    "Considerate",
    "Consoling",
    "Content",
    "Cooperative",
    "Courageous",
    "Courteous",
    "Creative",
    "Dashing",
    "Decent",
    "Delightful",
    "Dependable",
    "Devoted",
    "Diligent",
    "Dutiful",
    "Eager",
    "Echarming",
    "Empathetic",
    "Enchanting",
    "Endearing",
    "Energetic",
    "Engrossing",
    "Enjoyable",
    "Enjoyable",
    "Enterprising",
    "Enthusiastic",
    "Ethical",
    "Excited",
    "Extravagantly",
    "Faithful",
    "Fanciful",
    "Fantastic",
    "Fearless",
    "Fervent",
    "Funny",
    "Generous",
    "Gentle",
    "Genuine",
    "Giving",
    "Good-natured",
    "Happy",
    "Hard-working",
    "Helpful",
    "Heroic",
    "Honest",
    "Honorable",
    "Hospitable",
    "Humane",
    "Humorous",
    "Imaginary",
    "Imaginative",
    "Impassioned",
    "Industrious",
    "Ingenious",
    "Innovative",
    "Inspired",
    "Inspiring",
    "Intelligent",
    "Intrepid",
    "Inventive",
    "Joyous",
    "Keen",
    "Lavish",
    "Level-headed",
    "Liberal",
    "Likable",
    "Lively",
    "Lovely",
    "Loving",
    "Loyal",
    "Magnanimous",
    "Mindful",
    "Motivated",
    "Munificent",
    "Nice",
    "Novel",
    "Obliging",
    "Original",
    "Passionate",
    "Perceptive",
    "Persistent",
    "Personable",
    "Pioneering",
    "Pleasant",
    "Polite",
    "Popular",
    "Practical",
    "Pragmatic",
    "Profound",
    "Quick",
    "Quick-witted",
    "Rational",
    "Real",
    "Realistic",
    "Reliable",
    "Resourceful",
    "Respectful",
    "Responsible",
    "Riveting",
    "Secure",
    "Seductive",
    "Sensational",
    "Sensible",
    "Sharp",
    "Sincere",
    "Smart",
    "Solid",
    "Spirited",
    "Steadfast",
    "Steady",
    "Supportive",
    "Sympathetic",
    "Tenacious",
    "Tender",
    "Thorough",
    "Thoughtful",
    "True",
    "Trustworthy",
    "Truthful",
    "Unafraid",
    "Untroubled",
    "Upright",
    "Valiant",
    "Venturous",
    "Vigorous",
    "Virtuous",
    "Vivacious",
    "Well-behaved",
    "Wholehearted",
    "Witty",
]
